'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compileSimulation = exports.compileTabtest = undefined;

var _ramda = require('ramda');

var R = _interopRequireWildcard(_ramda);

var _btoa = require('btoa');

var _btoa2 = _interopRequireDefault(_btoa);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _xodFuncTools = require('xod-func-tools');

var _errorCodes = require('./errorCodes');

var EC = _interopRequireWildcard(_errorCodes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/* babel-plugin-inline-import 'xod-tabtest/cpp/Arduino.h' */ /* eslint-disable prettier/prettier */
var arduinoH = '\n#ifndef ARDUINO_H\n#define ARDUINO_H\n\n#include <algorithm>  // for min/max\n#include <cmath>      // for isfinite\n#include <stddef.h>   // for size_t\n#include <stdint.h>   // for uint32_t, etc\n#include <string.h>   // for strlen\n#include <stdlib.h>   // to bring `abs` before its redefinition below\n#include <math.h>\n\nvoid setup();\nvoid loop();\n\n#define A0 14\n\nusing ::std::min;\nusing ::std::max;\n\n// undefine C isfinite if encountered\n#ifdef isfinite\n#undef isfinite\n#endif\nusing ::std::isfinite;\n\n// undefine stdlib\'s abs if encountered\n// because all platforms\' Arduino.h do it\n#ifdef abs\n#undef abs\n#endif\n#define abs(x) ((x)>0?(x):-(x))\n\nvoid mockTime(uint32_t);\n\nuint32_t millis();\nuint32_t micros();\nvoid delay(uint32_t);\n\nclass Stream {\n  public:\n    void begin(uint32_t) {};\n    void end() {};\n\n    bool available() { return true; };\n    bool write(char) { return true; };\n    uint8_t read() { return 1; };\n    void flush() {};\n};\nclass SoftwareSerial : public Stream {};\nclass HardwareSerial : public Stream {};\n\n#endif\n';
/* babel-plugin-inline-import 'xod-tabtest/cpp/Arduino.cpp' */var arduinoCpp = '\n#include "Arduino.h"\n\nstatic uint32_t g_time = 1;\n\nvoid mockTime(uint32_t newTime) {\n    g_time = newTime;\n}\n\nuint32_t millis() {\n    return g_time;\n}\n\nuint32_t micros() {\n    return g_time * 1000;\n}\n\nvoid delay(uint32_t dt) {\n    // no-op\n}\n';
/* babel-plugin-inline-import '../../../cpplib/catch2utils/XStringFormat.inl' */var xStringFormatInl = '/**\n * This file makes Catch2 show failed tests properly for the XString type.\n * E.G.\n * "some" == "something"\n * instead of\n * {?} == {?}\n *\n * Catch2 string conversion docs:\n * https://github.com/catchorg/Catch2/blob/master/docs/tostring.md\n */\n\ntemplate <typename T>\nstruct XodStringMaker {\n    static std::string convert(T value) {\n        size_t len = length(value) + 2;\n        std::string str(len, \'\\0\');\n        size_t strPos = 1;\n\n        str[0] = \'"\';\n        str[len - 1] = \'"\';\n\n        for (auto it = value.iterate(); it; ++it) {\n            str[strPos++] = *it;\n        }\n        return str;\n    }\n};\n\nnamespace Catch {\ntemplate <>\nstruct StringMaker<xod::XString> {\n    static std::string convert(xod::XString value) {\n        return XodStringMaker<xod::XString>::convert(value);\n    }\n};\n\ntemplate <>\nstruct StringMaker<xod::XStringCString> {\n    static std::string convert(xod::XStringCString value) {\n        return XodStringMaker<xod::XStringCString>::convert(value);\n    }\n};\n}\n';


// :: String -> Nullable String -> StrMap Source -> Object -> Promise Object Error
var compile = R.curry(function (hostname, accessToken, suite, opts) {
  var reqUrl = 'https://api.' + hostname + '/compile/enqueue';
  var reqBody = R.compose(R.merge(opts), function (filesMap) {
    return {
      payload: filesMap
    };
  }, R.map(_btoa2.default), R.when(function () {
    return opts.fqbn === 'wasm:tabtest:2';
  }, R.merge({
    'Arduino.cpp': arduinoCpp,
    'Arduino.h': arduinoH,
    'XStringFormat.inl': xStringFormatInl
  })))(suite);

  var headers = R.when(function () {
    return (0, _xodFuncTools.notNil)(accessToken);
  }, R.assoc('Authorization', 'Bearer ' + accessToken))({
    'Content-Type': 'application/json'
  });

  return (0, _nodeFetch2.default)(reqUrl, {
    method: 'POST',
    headers: headers,
    body: JSON.stringify(reqBody)
  }).catch(function (err) {
    return Promise.reject((0, _xodFuncTools.createError)(EC.WASM_COMPILATION_RESULTS_FETCH_ERROR, { message: err.message }));
  }).then(function (res) {
    if (!res.ok) {
      return res.json().then(function (json) {
        switch (res.status) {
          case 401:
            return (0, _xodFuncTools.createError)(EC.WRONG_AUTHORIZATION_TOKEN, json);
          case 402:
            return (0, _xodFuncTools.createError)(EC.COMPILATION_LIMIT_EXCEEDED, json);
          case 422:
            return (0, _xodFuncTools.createError)(EC.WASM_COMPILATION_ERROR, json);
          default:
            return (0, _xodFuncTools.createError)(EC.WASM_UNKNOWN_COMPILATION_ERROR, json);
        }
      }).catch(function () {
        return (0, _xodFuncTools.createError)(EC.COMPILATION_SERVICE_ERROR, res.body);
      }).then(function (x) {
        return Promise.reject(x);
      });
    }
    return res;
  }).then(function (res) {
    return res.json();
  });
});

// :: String -> Nullable String -> StrMap Source -> Promise
var compileTabtest = exports.compileTabtest = R.curry(function (hostname, accessToken, suite) {
  return compile(hostname, accessToken, suite, {
    fqbn: 'wasm:tabtest:2',
    options: {}
  });
});

// :: String -> Nullable String -> Source -> Promise
var compileSimulation = exports.compileSimulation = R.curry(function (hostname, accessToken, programCode) {
  var suite = {
    'sketch.ino': programCode
  };
  return compile(hostname, accessToken, suite, {
    fqbn: 'wasm:simulation',
    options: {}
  });
});
//# sourceMappingURL=compile.js.map